// Control.java - creates and maintains the simulation control panel

import shared.model.structure.*;
import shared.CoordinateLayout.*;

import java.awt.event.*;
import java.awt.*;

import javax.swing.event.*;
import javax.swing.*;

import javax.vecmath.*;
import java.util.*;

public class Control
{
        JLabel timeUnitLabel;
        JLabel timeElapsedLabel;

        public Control()
        {
                //--- Initialise the control window ---//

                // Get the current look and feel
                try
		{
		    UIManager.setLookAndFeel(
				   UIManager.getSystemLookAndFeelClassName());
		} catch (Exception e) {}

                // Create a JFrame
		JFrame controlWindow = new JFrame("Protein Simulation Package - Control Panel");
                controlWindow.setSize(600, 300);
		controlWindow.getContentPane().setLayout(new BorderLayout());
		controlWindow.addWindowListener(new WindowAdapter()
                {
                        public void windowClosing(WindowEvent e) {System.exit(0);}
                });

                controlWindow.setJMenuBar(new mainMenu(controlWindow));

                // Add a start button
                final JButton start = new JButton("Start");
                start.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent e)
                        {
                                boolean state = !(main.getRunning());
                                main.setRunning(state);
                                if (state == false)
                                {
                                        start.setText("Start");
                                }
                                else
                                {
                                        start.setText("Stop");
                                }

                        }
                });

                // Create the timestep selector
                timeUnitLabel = new JLabel("");
                timeElapsedLabel = new JLabel("0.0");
                TimeStepUnitMenu tuMenu = new TimeStepUnitMenu(timeUnitLabel);

                // Create a JPanel for the objects
                JPanel mainPanel = new JPanel();
                mainPanel.setPreferredSize(new Dimension(600, 300));

                mainPanel.setLayout(new CoordinateLayout(new Dimension(10, 10), 2, 2));

		mainPanel.add(start, new Coordinate(4,7,6,8));
                mainPanel.add(tuMenu, new Coordinate(4,1,6,2));
                mainPanel.add(timeUnitLabel, new Coordinate(4,2,6,3));
                mainPanel.add(timeElapsedLabel, new Coordinate(2,2,4,3));

                controlWindow.getContentPane().add(mainPanel);

                controlWindow.setVisible(true);
        }

        public void setElapsedTime(float et)
        {
                timeElapsedLabel.setText(""+(et / main.getTimeUnit()));
        }
}


class TimeStepUnitMenu extends JComboBox implements ItemListener
{
	JLabel simElapseUnit;

	public TimeStepUnitMenu(JLabel seu)
	{
		super();
		simElapseUnit = seu;

		addItem("Seconds");
		addItem("Milliseconds");
		addItem("Microseconds");
		addItem("Nanoseconds");
		addItem("Picoseconds");
		addItem("Femtoseconds");

		setEditable(false);

		setSelectedIndex(2);
                simElapseUnit.setText("Microseconds");
		main.setTimeUnit((float) Math.pow(10.0, -6.0));

		addItemListener(this);
	}

	public void itemStateChanged(ItemEvent ie)
	{
		String s = (String) ie.getItem();

		if (s.equals("Seconds")) { main.setTimeUnit(1f); }
		if (s.equals("Milliseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Microseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -6.0)); }
		if (s.equals("Nanoseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -9.0)); }
		if (s.equals("Picoseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -12.0)); }
		if (s.equals("Femtoseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -15.0)); }

		simElapseUnit.setText(s);
                main.setElapsedTime();
	}

	public void refresh()
	{
		String s = (String) getSelectedItem();

		if (s.equals("Seconds")) { main.setTimeUnit(1f); }
		if (s.equals("Milliseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -3.0)); }
		if (s.equals("Microseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -6.0)); }
		if (s.equals("Nanoseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -9.0)); }
		if (s.equals("Picoseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -12.0)); }
		if (s.equals("Femtoseconds"))
			{ main.setTimeUnit((float) Math.pow(10.0, -15.0)); }
	}

}

class TimeStepValueTextField extends JTextField implements KeyListener
{

	public TimeStepValueTextField()
	{
		super(6);
		setText("1");
		main.setTimeValue(1f);
		setHorizontalAlignment(JTextField.RIGHT);
		addKeyListener(this);
	}

	public void keyPressed(KeyEvent ke)
	{}

	public void keyTyped(KeyEvent ke)
	{}

	public void keyReleased(KeyEvent ke)
	{
		try
		{
			float f = (float) (Float.parseFloat(getText()));
			main.setTimeValue(f);
		}
		catch (NumberFormatException e)
		{}
	}
}