package shared.CoordinateLayout;

import java.awt.*;
import java.util.*;

public class CoordinateLayout implements LayoutManager2 
{
    int horizontalPadding, verticalPadding;

    Hashtable coordHash;
    Dimension maxCoord;

    public CoordinateLayout(Dimension d, int h, int v) 
	{
        horizontalPadding = h;
        verticalPadding = v;

        coordHash = new Hashtable();
        maxCoord = new Dimension(d);
    }
    
	// LayoutManager method overrides 

    public void addLayoutComponent(String s, Component c) {}

    public void removeLayoutComponent(Component c) 
	{
        coordHash.remove(c);
    }

	public Dimension preferredLayoutSize(Container c) 
	{
		int i = Integer.MAX_VALUE;
        return new Dimension(i,i);//getLayoutSize(c, true);
    }

    public Dimension minimumLayoutSize(Container c) 
	{
        return new Dimension(0,0);//getLayoutSize(c, false);
    }
	
	public void layoutContainer(Container container) 
	{
		float adjustedWidth, adjustedHeight;
		float gridSquareX, gridSquareY;
		int x1, y1, width, height;

		Object lock = container.getTreeLock();
        synchronized(lock) 
		{
            int componentcount = container.getComponentCount();
            if (componentcount==0) {return;}

            Dimension size = container.getSize();
            Insets ins = container.getInsets();

            gridSquareX = (float) ((size.width-ins.right-ins.left+horizontalPadding)/maxCoord.getWidth());
            gridSquareY = (float) ((size.height-ins.top-ins.bottom+verticalPadding)/maxCoord.getHeight());
                
            for (int counter=0; counter<componentcount; counter++) 
			{
                Component currentcomponent = container.getComponent(counter);
                Coordinate coords = (Coordinate)coordHash.get(currentcomponent);
               
                x1 = ins.left + (int)((float)coords.x1()*gridSquareX);
                y1 = ins.top + (int)((float)coords.y1()*gridSquareY);

                width = (int)((float)coords.getWidth()*gridSquareX)-horizontalPadding;
                height = (int)((float)coords.getHeight()*gridSquareY)-verticalPadding;

                currentcomponent.setBounds(x1, y1, width, height);
            }
        }
    }

	// LayoutManager2 method overrides 

    public void addLayoutComponent(Component c, Object obj) 
	{
        Coordinate coords = (Coordinate)obj;
        addToHash(c, coords);
    }

    public float getLayoutAlignmentX(Container c) 
	{
        return (float) 0.5;
    }
	
    public float getLayoutAlignmentY(Container c) 
	{
        return (float) 0.5;
	}

	public void invalidateLayout(Container c) {}

	public Dimension maximumLayoutSize(Container c) 
	{
		int i = Integer.MAX_VALUE;
        return (new Dimension(i,i));
    }

    public void addToHash(Component c, Coordinate coords) 
	{
        coordHash.put(c, coords);
    }
}