package shared.model.reader;

import neuroml.core.*;
import neuroml.util.*;

import java.util.*;

import shared.model.structure.*;
import shared.model.reader.xmlobjects.XMLParticleTemplate;
import shared.model.reader.xmlobjects.*;
import shared.model.reader.XMLList;

public class FileHandler
{
   public FileHandler()
   {
   }

   /////////// Output //////////////

   public static void toFile(String filename, Vector modelVect)
   {
        Vector xmlVect = convertToXMLObjects(modelVect);

        XMLOut.toFile(new XMLList(xmlVect), filename);
   }

   private static Vector convertToXMLObjects(Vector modelVect)
   {
        Vector xmlVect = new Vector();

        for (int i=0; i<modelVect.size(); i++)
        {
                Object obj = modelVect.elementAt(i);

                if (obj instanceof ParticleTemplate)
                {
                        xmlVect.addElement(new XMLParticleTemplate((ParticleTemplate) obj));
                }
                else if (obj instanceof SimObjectTemplate)
                {
                        xmlVect.addElement(new XMLSimObjectTemplate((SimObjectTemplate) obj));
                }
                else if (obj instanceof SimObject)
                {
                        xmlVect.addElement(new XMLSimObject((SimObject) obj));
                }
                else if (obj instanceof InteractionTemplate)
                {
                        xmlVect.addElement(new XMLInteractionTemplate((InteractionTemplate) obj));
                }
                else if (obj instanceof StateTemplate)
                {
                        xmlVect.addElement(new XMLStateTemplate((StateTemplate) obj));
                }
                else if (obj instanceof PotentialTemplate)
                {
                        xmlVect.addElement(new XMLPotentialTemplate((PotentialTemplate) obj));
                }
        }

        return xmlVect;
   }
   ////////// Input ///////////

   // load in the specified file
   public static Vector fromFile(String filename)
   {
        XMLIn.addKnownPackage("shared");
        XMLIn.addKnownPackage("shared.model");
        XMLIn.addKnownPackage("shared.model.reader");
        XMLIn.addKnownPackage("shared.model.reader.xmlobjects");
        XMLList xmlList = (XMLList) XMLIn.loadXML(filename);

        Vector xmlVect = xmlList.toVector();

        return convertFromXMLObjects(xmlVect);
   }

   private static Vector convertFromXMLObjects(Vector xmlVect)
   {
        Vector modelVect = new Vector();

        for (int i=0; i<xmlVect.size(); i++)
        {
                Object xmlObj = xmlVect.elementAt(i);
                if (xmlObj instanceof XMLParticleTemplate)
                {
                        ParticleTemplate pt =(ParticleTemplate) (((XMLParticleTemplate)xmlObj).getObject());
                        modelVect.addElement(pt);
                }
                else if (xmlObj instanceof XMLSimObjectTemplate)
                {
                        SimObjectTemplate sot =(SimObjectTemplate) (((XMLSimObjectTemplate)xmlObj).getObject());
                        modelVect.addElement(sot);
                }
                else if (xmlObj instanceof XMLSimObject)
                {
                        SimObject so =(SimObject) (((XMLSimObject)xmlObj).getObject());
                        modelVect.addElement(so);
                }
                else if (xmlObj instanceof XMLInteractionTemplate)
                {
                        InteractionTemplate it =(InteractionTemplate) (((XMLInteractionTemplate)xmlObj).getObject());
                        modelVect.addElement(it);
                }
                else if (xmlObj instanceof XMLStateTemplate)
                {
                        StateTemplate st = (StateTemplate) (((XMLStateTemplate) xmlObj).getObject());
                        modelVect.addElement(st);
                }
                else if (xmlObj instanceof XMLPotentialTemplate)
                {
                        PotentialTemplate pt = (PotentialTemplate) (((XMLPotentialTemplate) xmlObj).getObject());
                        modelVect.addElement(pt);
                }
        }

        return modelVect;
   }
}
