package shared.model.reader.xmlobjects;

import neuroml.core.*;

import shared.model.structure.*;
import shared.model.utils.*;

import java.util.*;
import javax.vecmath.*;

public class XMLSimObjectTemplate
{
        public String id;

        public String particleTemplateIds = "";
        public neuroml.core.List particleTemplateCoordX = new neuroml.core.List();
        public neuroml.core.List particleTemplateCoordY = new neuroml.core.List();
        public neuroml.core.List particleTemplateCoordZ = new neuroml.core.List();
        public neuroml.core.List particleTemplateOrientX = new neuroml.core.List();
        public neuroml.core.List particleTemplateOrientY = new neuroml.core.List();
        public neuroml.core.List particleTemplateOrientZ = new neuroml.core.List();
        public neuroml.core.List particleTemplateOrientAngle = new neuroml.core.List();

        public String subObjectTemplateIds = "";
        public neuroml.core.List subObjectTemplateCoordX = new neuroml.core.List();
        public neuroml.core.List subObjectTemplateCoordY = new neuroml.core.List();
        public neuroml.core.List subObjectTemplateCoordZ = new neuroml.core.List();
        public neuroml.core.List subObjectTemplateOrientX = new neuroml.core.List();
        public neuroml.core.List subObjectTemplateOrientY = new neuroml.core.List();
        public neuroml.core.List subObjectTemplateOrientZ = new neuroml.core.List();
        public neuroml.core.List subObjectTemplateOrientAngle = new neuroml.core.List();

        public String stateTemplateIds = "";

        public XMLSimObjectTemplate()
        {}

        public XMLSimObjectTemplate(SimObjectTemplate template)
        {
                id = template.getId();
                String[] particleIds = template.getParticleTemplateIds();
                Vector3f[] particleCOMs = template.getParticleCentreOfMasses();
                AxisAngle4f[] particleOrient = template.getParticleOrientations();

                for (int i=0; i<particleIds.length; i++)
                {
                        particleTemplateIds += ";" + particleIds[i];
                        Vector3f com = particleCOMs[i];
                        particleTemplateCoordX.add(new Float(com.x));
                        particleTemplateCoordY.add(new Float(com.y));
                        particleTemplateCoordZ.add(new Float(com.z));
                        AxisAngle4f orient = particleOrient[i];
                        particleTemplateOrientX.add(new Float(orient.x));
                        particleTemplateOrientY.add(new Float(orient.y));
                        particleTemplateOrientZ.add(new Float(orient.z));
                        particleTemplateOrientAngle.add(new Float(orient.angle));
                }
                // strip the leading colon from particleTemplateIds
                if (particleTemplateIds.length() > 0)
                {
                        particleTemplateIds = particleTemplateIds.substring(1);
                }

                String[] subObjectIds = template.getSubObjectTemplateIds();
                Vector3f[] subObjectCOMs = template.getSubObjectCentreOfMasses();
                AxisAngle4f[] subObjectOrient = template.getSubObjectOrientations();

                for (int i=0; i<subObjectIds.length; i++)
                {
                        subObjectTemplateIds += ";" + subObjectIds[i];
                        Vector3f scom = subObjectCOMs[i];
                        subObjectTemplateCoordX.add(new Float(scom.x));
                        subObjectTemplateCoordY.add(new Float(scom.y));
                        subObjectTemplateCoordZ.add(new Float(scom.z));
                        AxisAngle4f sorient = subObjectOrient[i];
                        subObjectTemplateOrientX.add(new Float(sorient.x));
                        subObjectTemplateOrientY.add(new Float(sorient.y));
                        subObjectTemplateOrientZ.add(new Float(sorient.z));
                        subObjectTemplateOrientAngle.add(new Float(sorient.angle));
                }
                // strip the leading colon from particleTemplateIds
                if (subObjectTemplateIds.length() > 0)
                {
                        subObjectTemplateIds = subObjectTemplateIds.substring(1);
                }

                String[] stateIds = template.getStateTemplateIds();
                for (int i=0; i<stateIds.length; i++)
                {
                        stateTemplateIds += ";" + stateIds[i];
                }

                if (stateTemplateIds.length() > 0)
                {
                        stateTemplateIds = stateTemplateIds.substring(1);
                }
        }

        public SimObjectTemplate getObject()
        {
                SimObjectTemplate template = new SimObjectTemplate(id);

                String[] particleTemplates = Functions.split(';', particleTemplateIds);
                Object[] xcoord = particleTemplateCoordX.toArray();
                Object[] ycoord = particleTemplateCoordY.toArray();
                Object[] zcoord = particleTemplateCoordZ.toArray();

                Object[] xorient = particleTemplateOrientX.toArray();
                Object[] yorient = particleTemplateOrientY.toArray();
                Object[] zorient = particleTemplateOrientZ.toArray();
                Object[] angorient = particleTemplateOrientAngle.toArray();

                Vector3f[] com = new Vector3f[particleTemplates.length];
                AxisAngle4f[] orient = new AxisAngle4f[particleTemplates.length];

                for (int i=0; i<particleTemplates.length; i++)
                {
                        com[i] = new Vector3f(((Float)(xcoord[i])).floatValue(),
                                                    ((Float)(ycoord[i])).floatValue(),
                                                    ((Float)(zcoord[i])).floatValue());

                        orient[i] = new AxisAngle4f(((Float)(xorient[i])).floatValue(),
                                                             ((Float)(yorient[i])).floatValue(),
                                                             ((Float)(zorient[i])).floatValue(),
                                                             ((Float)(angorient[i])).floatValue());
                }

                template.setParticles(particleTemplates, com, orient);

                String[] objectTemplates = Functions.split(';', subObjectTemplateIds);
                xcoord = subObjectTemplateCoordX.toArray();
                ycoord = subObjectTemplateCoordY.toArray();
                zcoord = subObjectTemplateCoordZ.toArray();

                xorient = subObjectTemplateOrientX.toArray();
                yorient = subObjectTemplateOrientY.toArray();
                zorient = subObjectTemplateOrientZ.toArray();
                angorient = subObjectTemplateOrientAngle.toArray();

                com = new Vector3f[objectTemplates.length];
                orient = new AxisAngle4f[objectTemplates.length];

                for (int i=0; i<objectTemplates.length; i++)
                {
                        com[i] = new Vector3f(((Float)(xcoord[i])).floatValue(),
                                                    ((Float)(ycoord[i])).floatValue(),
                                                    ((Float)(zcoord[i])).floatValue());

                        orient[i] = new AxisAngle4f(((Float)(xorient[i])).floatValue(),
                                                             ((Float)(yorient[i])).floatValue(),
                                                             ((Float)(zorient[i])).floatValue(),
                                                             ((Float)(angorient[i])).floatValue());
                }

                template.setSubObjects(objectTemplates, com, orient);

                // Set the state information for the template
                String[] stateIds = Functions.split(';', stateTemplateIds);
                template.setStateTemplateIds(stateIds);

                return template;
        }
}